@@ -2,7 +2,7 @@ use colored::Colorize;
 use log::error;
 use nix::sys::signal;
 use nix::unistd;
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::ffi::OsStr;
 use std::io;
 use std::process::Stdio;
@@ -139,17 +139,44 @@ enum CommandFrame {
     Error(io::Error),
 }
 
+#[derive(Default, Clone)]
+pub struct Options<'a> {
+    environment: HashMap<&'a str, &'a str>,
+    tki: Option<Duration>,
+    input_data: Option<&'a [u8]>,
+}
+
+impl<'a> Options<'a> {
+    #[inline]
+    pub fn new() -> Self {
+        Self::default()
+    }
+    #[inline]
+    pub fn tki(mut self, t: Duration) -> Self {
+        self.tki.replace(t);
+        self
+    }
+    #[inline]
+    pub fn input_data(mut self, data: &'a [u8]) -> Self {
+        self.input_data.replace(data);
+        self
+    }
+    #[inline]
+    pub fn env(&mut self, name: &'a str, value: &'a str) {
+        self.environment.insert(name, value);
+    }
+}
+
 #[allow(clippy::too_many_lines)]
 #[allow(clippy::missing_panics_doc)]
 /// # Errors
 ///
 /// Will return `Err` on I/O errors
-pub async fn command<I, S>(
+pub async fn command<'a, I, S>(
     command: &str,
     args: I,
     timeout: Duration,
-    tki: Option<Duration>,
-    data: Option<String>,
+    opts: Options<'a>,
 ) -> Result<CommandResult, io::Error>
 where
     I: IntoIterator<Item = S>,
@@ -161,8 +188,9 @@ where
         .stderr(Stdio::piped())
         .kill_on_drop(true)
         .args(args)
+        .envs(opts.environment)
         .spawn()?;
-    let stdin = if data.is_some() {
+    let stdin = if opts.input_data.is_some() {
         match child.stdin.take() {
             Some(v) => Some(v),
             None => {
@@ -218,18 +246,16 @@ where
         task::spawn(async move {
             sleep(timeout).await;
             #[allow(clippy::cast_possible_wrap)]
-            kill_pstree(pid, tki, true).await;
+            kill_pstree(pid, opts.tki, true).await;
             let _r = tx_guard.send(CommandFrame::Terminated).await;
         })
     });
-    let fut_stdin = stdin_writer.map(|mut writer| {
-        task::spawn(async move {
-            if let Err(e) = writer.write_all(data.unwrap().as_bytes()).await {
-                error!("Unable to write to stdin: {}", e);
-            } else if let Err(e) = writer.flush().await {
-                error!("Unable to flush stdin: {}", e);
-            }
-        })
+    stdin_writer.map(|mut writer| async move {
+        if let Err(e) = writer.write_all(opts.input_data.unwrap()).await {
+            error!("Unable to write to stdin: {}", e);
+        } else if let Err(e) = writer.flush().await {
+            error!("Unable to flush stdin: {}", e);
+        }
     });
     let fut_stdout = task::spawn(async move {
         while let Some(line) = match stdout_reader.next_line().await {
@@ -273,9 +299,6 @@ where
             }
             CommandFrame::Terminated => {
                 runner.abort();
-                if let Some(f) = fut_stdin {
-                    f.abort();
-                }
                 fut_stdout.abort();
                 fut_stderr.abort();
                 return Ok(result);
@@ -285,14 +308,11 @@ where
                 if let Some(g) = guard {
                     g.abort();
                 }
-                if let Some(f) = fut_stdin {
-                    f.abort();
-                }
                 fut_stdout.abort();
                 fut_stderr.abort();
                 #[allow(clippy::cast_possible_wrap)]
                 ppid.map(|pid| async move {
-                    kill_pstree(pid, tki, true).await;
+                    kill_pstree(pid, opts.tki, true).await;
                 });
                 return Err(e);
             }
